function [plan, rawFeatures, rawLabelInts, resultDumpFilename] = ...
    alpha1vs2_prepareCaseByName(nameOfCase)
% Prepares one of our test cases. These are separated because we store
% loads of numerical data for each. It turns out that not everything
% is needed, but we'll deal with that better in later research...

% Must have a name as string
if (nargin<1) || (~ischar(nameOfCase))
    error('Parameter must be the name of an experiment.')
end;

%% Load original data
% The name must be one of the cases reported in our final WCCI paper.
if strcmp(nameOfCase, 'iris')
    dataFileName = fullfile(alpha1vs2_dirsetup('ucidir'),'iris_numerical.dat');
    [rawFeatures, rawLabelInts] = loadLabeledData(dataFileName, 150, 4+1, 5);
elseif strcmp(nameOfCase, 'wine')
    dataFileName = fullfile(alpha1vs2_dirsetup('ucidir'),'wine.data');
    [rawFeatures, rawLabelInts] = loadLabeledData(dataFileName, 59+71+48, 13+1, 1);
elseif strcmp(nameOfCase, 'vowel')
    dataFileName = fullfile(alpha1vs2_dirsetup('ucidir'),'vowel_commasep.dat');
    [rawFeatures, rawLabelInts] = loadLabeledData(dataFileName, 90*11, 10+1, 11);
elseif strcmp(nameOfCase, 'pen10')
    % For the Pendigits cases, we only take the original training data!
    dataFileName = fullfile(alpha1vs2_dirsetup('ucidir'),'pendigits.tra');
    [rawFeatures, rawLabelInts] = loadLabeledData(dataFileName, 7494, 16+1, 17);
elseif strcmp(nameOfCase, 'pen5')
    % (See the note on pen10)
    dataFileName = fullfile(alpha1vs2_dirsetup('ucidir'),'pendigits.tra');
    [rawFeatures, rawLabelInts] = loadLabeledData(dataFileName, 7494, 16+1, 17);
    % For Pen5, take only some digits to see if results are different for 
    % smaller number of classes:
    % We made the selection once, and then screwed up with storing random states, 
    % so let us load the same permutation for both half and quarter size case:
    load pen5_indReducedSet.mat;
    indReducedSet = pen5_indShuffledReducedSet(1:end); %indReducedSet = find(rawLabelInts <= 5);
    rawFeatures = rawFeatures(:,indReducedSet);
    rawLabelInts = rawLabelInts(:,indReducedSet);
elseif strcmp(nameOfCase, 'pen5h')
    % (See the notes on pen5)
    dataFileName = fullfile(alpha1vs2_dirsetup('ucidir'),'pendigits.tra');
    [rawFeatures, rawLabelInts] = loadLabeledData(dataFileName, 7494, 16+1, 17);
    load pen5_indReducedSet.mat;
    indReducedSet = pen5_indShuffledReducedSet(1:end/2);  % Half-sized
    rawFeatures = rawFeatures(:,indReducedSet);
    rawLabelInts = rawLabelInts(:,indReducedSet);
elseif strcmp(nameOfCase, 'pen5q')
    % (See the notes on pen5)
    dataFileName = fullfile(alpha1vs2_dirsetup('ucidir'),'pendigits.tra');
    [rawFeatures, rawLabelInts] = loadLabeledData(dataFileName, 7494, 16+1, 17);
    load pen5_indReducedSet.mat;
    indReducedSet = pen5_indShuffledReducedSet(1:end/4);  % Quarter-sized
    rawFeatures = rawFeatures(:,indReducedSet);
    rawLabelInts = rawLabelInts(:,indReducedSet);
else
    error('This set of experiments did not consider anything by the name "%s"!',nameOfCase);
end

nclasses = max(rawLabelInts);

%% Plan, defaults for most cases
% This is the same for most of the cases. Note the few differences in the
% if-blocks to follow, though!
plan.nperms = 15;                   % Generated datasets by rnd division
plan.nruns = 20;                    % Starting points in multistart optimization
plan.planDirty = [.0 .10 .20 .30];  % Label noise ratios to experiment.
plan.planIters = [0 100 200 200 500 500 500 500 500 500 500]; % Restart checkpoints
plan.rTrainSet = .50;               % Portion of samples to use in training

% After many experiments, we ended up in some final selections for all cases:
plan.dirtifyClasses=1:nclasses;     % Dirtify all (1:nclasses) classes
plan.dirtify = 'targets';           % Dirtify targets instead of inputs
plan.prototypes = -1 + 2*eye(nclasses);  % Binary coding of classes
plan.planRegc = [.001];             % Small, to help optimizer 
                                    % but play no big role otherwise.

% For most cases, the MLP has one hidden layer with nclasses + 3 neurons)
plan.nnHiddenArch = [ nclasses+3 ]; 

%% Exceptions to the plan for some experiments
if strcmp(nameOfCase, 'pen10') ...
        || strcmp(nameOfCase, 'pen5') ...
        || strcmp(nameOfCase, 'pen5h') ...
        || strcmp(nameOfCase, 'pen5q')
    plan.nperms = 5;      % To keep computation time reasonable
elseif strcmp(nameOfCase, 'vowel')
    plan.nperms = 5;      % To keep computation time reasonable
    plan.rTrainSet = .75; % To get better classification overall
    plan.nnHiddenArch = [ 20 nclasses+3 ];   % Complex data -> complex net
end

%% Result dump filename that contains a time stamp
% Current time with seconds is accurate enough for our current purposes.
resultDumpFilename = fullfile(alpha1vs2_dirsetup('tempdir'), ...
    sprintf('Matlab_res_%s_s%s.mat',nameOfCase,datestr(now,'yyyymmddTHHMMSS')));
