function net = demo_iris(netCont)
% Trying the codes on the Iris data, which is a bit of a classic.
% This is quite an easy one, with very fast training and good result except
% for a couple of tricky flowers that are very hard to classify.
%
% This will print out a so-called confusion matrix after each few steps of
% training. From there you will see how the MLP gets things first into
% wrong classes, and then, first fast and later rather one-by-one, the
% items fall to their true classes. Except for the painfully tricky ones.

% For training we need input vectors and corresponding target vectors.
datafile = fullfile(alpha1vs2_dirsetup('ucidir'), 'iris_numerical.dat');
dataN = 150;   dataD = 4+1;   labelI = 5;   nclasses = 3;


%% Encode targets. 

%prototypes = -1 + 2*eye(3);              % Binary coding (N/A for 2d demo)
prototypes = createCirclePoints(nclasses); % On a 2d circle rim

[vectors,labels] = loadLabeledData(datafile, dataN, dataD, labelI, prototypes);

%% Preprocess

% There can be some order in a data file, like in the iris. First thing to
% do is to make a random (or better) permutation before we apply methods, 
% or split the data set. Here's a random one:
oldstate = rand('state');
rand('state', 1234);
p = randperm(size(vectors,2));
vectors = vectors(:,p);
labels = labels(:,p);
rand('state', oldstate);

% Let's split the data for training and testing (50%/50% for the demo):
nt = floor(.5*size(vectors,2));
input   = vectors(:,1:nt);
target  =  labels(:,1:nt);
testx   = vectors(:,nt+1:end);
trueOut =  labels(:,nt+1:end);

% Simulate wrong labels in training data; just to see how it
% affects either the mean or spatial median training..
wrongRatio = .0 %.15
nwrong = floor(wrongRatio*size(input,2));
p = randperm(size(input,2));
target(:,1:nwrong) = target(:,p(1:nwrong));


% Scale to [-1,1]:
[input, traA, traB] = scaleCols(input,[-1 1]);

% In principle, testing data is "unforeseen", so we can only scale it based
% on the knowledge gathered from the training set:
testx = scaleCols(testx,traA,traB);

%disp(input(:,1:4)); disp(target(:,1:4)); disp(testx(:,1:4)); disp(trueOut(:,1:4))

%% Make the demo visualization

% Initialize figures.
figOriginal = figure; %fig or axes?
figOverlay = figure;

% Pick random initial weights, and optimize from there:
net = simple_mlp_create([size(input,1) 10 size(target,1)]);
net.regc = 0.6;
net.alpha = 2;

if nargin==1, net = netCont; end;

uFunc = @simple_mlp_evalMSErVecWrap;

extraPauseDuration=2; % pause for slower demonstration.
atOnce = 50;
for s = 1:20

  uPar.net = net;
  uPar.inputs = input;
  uPar.targets = target;

  x = simple_mlp_cell2vec(net.W);
%  x = simple_mlp_optimGD(x, uFunc, uPar, 0.05, atOnce, 0, 1e-6);
  x = simple_mlp_optimCGP(x, uFunc, uPar, 1, atOnce, 0, 1e-6);
%  x = simple_mlp_optimWrapSolvOpt(x, uFunc, uPar, 1, atOnce, 0, 1e-6);

  net.W = simple_mlp_vec2cell(x, net.nneur);

  % Evaluate and print accuracy measures.
  f = simple_mlp_evalMSEr(net, input, target);

  trainOut = simple_mlp_evalMSEr(net, input, 'feed');
  trainRate = classifAccuracy(trainOut, target, prototypes);
  
  testOut = simple_mlp_evalMSEr(net, testx, 'feed');
  [rate, cm] = classifAccuracy(testOut, trueOut, prototypes);
  
  %confusionMatrix_print(cm, {'c1','c2','c3'});
  commentary = sprintf('Iter %d: Cost=%f, TrainRate=%f TestRate=%f', ...
		       s*atOnce, f, trainRate, rate);
  disp(commentary);

  
  [dummy, trainClasses] = classifToClosest(trainOut, prototypes);
  [dummy, testClasses] = classifToClosest(testOut, prototypes);
  [dummy, trueClasses] = classifToClosest(trueOut, prototypes);

  % A hack:
  for ic = 1:3
    visTestOut(:,trueClasses==ic) = testOut(:,trueClasses==ic) - ...
	repmat(prototypes(:,ic),1,sum(trueClasses==ic)); % sic

    visTrueOut(:,trueClasses==ic) = trueOut(:,trueClasses==ic) - ...
	repmat(prototypes(:,ic),1,sum(trueClasses==ic));

    indSel = trainClasses==ic;
    if sum(indSel)
      visTrainOut(:,indSel) = trainOut(:,indSel) - ...
	  repmat(prototypes(:,ic),1,sum(indSel));
    else
      visTrainOut=[]
    end
  end

  figure(figOriginal);
  classvisual2d(testOut, testClasses, trueClasses, ...
		prototypes, ... 
		trainOut, trainClasses, target, {'c1' 'c2' 'c3'});
  set(gca, 'DataAspectRatio',[1 1 1]);
  
  figure(figOverlay);
  classvisual2d(visTestOut, testClasses, trueClasses, ...
                zeros(size(prototypes)), ...
		        visTrainOut, trainClasses, target, {'c1' 'c2' 'c3'});
  set(gca, 'DataAspectRatio',[1 1 1]);

  disp('Mean of error vectors:');
  disp(mean(visTrainOut,2));

  pause(0.5);
  
  for ic=1:3
    outs = trainOut(:,trainClasses == ic);
    center(:,ic) = sum(outs,2) ./ size(outs,2);
    offs(:,ic) = center(:,ic) - prototypes(:,ic);
  end
  %  offsNorms = sqrt(sum((trainOut - trueOut) .^ 2, 1));
  offsNorms = sqrt(sum(offs .^ 2, 1));
  disp(offsNorms);
%  disp(sprintf('mean err = %f    median err = %f', mean(trainErrNorms), ...
%	       median(trainErrNorms)));
  pause(extraPauseDuration);
 
end
